/* draw_wave
 * Copyright (C) 2012 Joël Krähemann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <math.h>
#include <stdlib.h>

static cairo_surface_t *sine_surface = NULL;
static cairo_surface_t *cosine_surface = NULL;

static unsigned int RED_PIXEL = 255 << 16;
static unsigned int GREEN_PIXEL = 255 << 8;
static unsigned int BLUE_PIXEL = 255;
static unsigned int WHITE_PIXEL;

static int STRIDE;

void
window_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  /* leave main loop */
  gtk_main_quit();
}

void
sine_drawing_area_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  GdkWindow *window;
  cairo_t *cr;

  /* create cairo context */
  window = gtk_widget_get_window(widget);
  cr = gdk_cairo_create(window);

  /* do painting */
  cairo_set_source_surface(cr, sine_surface, 0, 0);
  cairo_surface_mark_dirty(sine_surface);
  cairo_paint(cr);
}

void
cosine_drawing_area_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  GdkWindow *window;
  cairo_t *cr;

  /* create cairo context */
  window = gtk_widget_get_window(widget);
  cr = gdk_cairo_create(window);

  /* do painting */
  cairo_set_source_surface(cr, cosine_surface, 0, 0);
  cairo_surface_mark_dirty(cosine_surface);
  cairo_paint(cr);
}

void
putpixel(unsigned char *data, int x, int y, unsigned long int pixel)
{
  int bpp = 4; // STRIDE / 400;
  /* Here p is the address to the pixel we want to set */
  unsigned char *p = data + y * STRIDE + x * bpp;

  switch(bpp) {
  case 1:
    *p = pixel;
    break;

  case 2:
    *(unsigned short int *)p = pixel;
    break;

  case 3:
      p[0] = pixel & 0xff;
      p[1] = (pixel >> 8) & 0xff;
      p[2] = (pixel >> 16) & 0xff;
    break;

  case 4:
    *(unsigned long int *)p = pixel;
    break;
  }
}

void
draw_sine(cairo_surface_t *surface)
{
  unsigned char *data;
  double value;
  int offset;
  int i;

  data = cairo_image_surface_get_data(surface);

  for(i = 0; i < 400; i++){
    value = sin(((double) i / 400.0) * 2.0 * M_PI - M_PI) * 100.0;
    

    putpixel(data, i, 100 + (int) floor(value), WHITE_PIXEL);
  }

  cairo_surface_flush(surface);
}

void
draw_cosine(cairo_surface_t *surface)
{
  unsigned char *data;
  double value;
  int offset;
  int i;

  data = cairo_image_surface_get_data(surface);

  for(i = 0; i < 400; i++){
    value = cos(((double) i / 400.0) * 2.0 * M_PI + M_PI) * 100.0;

    putpixel(data, i, 100 + (int) floor(value), WHITE_PIXEL);
  }

  cairo_surface_flush(surface);
}

int
main(int argc, char **argv)
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *sine_drawing_area;
  GtkWidget *cosine_drawing_area;

  gtk_init(&argc, &argv);

  WHITE_PIXEL = RED_PIXEL | GREEN_PIXEL | BLUE_PIXEL;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  g_signal_connect(G_OBJECT(window), "delete-event\0",
		   G_CALLBACK(window_delete_event), NULL);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window),
		    vbox);
  /* sine related */
  sine_surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, 400, 200);
  STRIDE = cairo_image_surface_get_stride(sine_surface);

  draw_sine(sine_surface);

  sine_drawing_area = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(sine_drawing_area), 400, 200);
  gtk_box_pack_start(GTK_BOX(vbox),
		     sine_drawing_area,
		     FALSE, FALSE,
		     0);
  g_signal_connect(G_OBJECT(sine_drawing_area), "expose_event\0",
		   G_CALLBACK(sine_drawing_area_expose_event), NULL);

  /* cosine related */
  cosine_surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, 400, 200);

  draw_cosine(cosine_surface);

  cosine_drawing_area = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(cosine_drawing_area), 400, 200);
  gtk_box_pack_start(GTK_BOX(vbox),
		     cosine_drawing_area,
		     FALSE, FALSE,
		     0);
  g_signal_connect(G_OBJECT(cosine_drawing_area), "expose_event\0",
		   G_CALLBACK(cosine_drawing_area_expose_event), NULL);

  /* show all widgets */
  gtk_widget_show_all(window);

  /* enter main loop */
  gtk_main();

  return(0);
}
